from skidl import Pin, Part, SchLib, SKIDL, TEMPLATE

SKIDL_lib_version = '0.0.1'

microchip_pic12mcu = SchLib(tool=SKIDL).add_parts(*[
        Part(name='PIC12(L)F1501-I/P',dest=TEMPLATE,tool=SKIDL,do_erc=True,aliases=['PIC12(L)F1501-I/SN', 'PIC12(L)F1501-I/MS', 'PIC12(L)F1501-I/MC']),
        Part(name='PIC12(L)F1822-I/P',dest=TEMPLATE,tool=SKIDL,do_erc=True,aliases=['PIC12(L)F1822-I/SN', 'PIC12(L)F1822-I/MC']),
        Part(name='PIC12(L)F1840-I/P',dest=TEMPLATE,tool=SKIDL,do_erc=True,aliases=['PIC12(L)F1840-I/SN', 'PIC12(L)F1840-I/MC']),
        Part(name='PIC12C508-I/P',dest=TEMPLATE,tool=SKIDL,keywords='8-Bit CMOS Microcontroller',description='PIC12C508, 512W EPROM, 25B SRAM, SO8 Wide',ref_prefix='U',num_units=1,do_erc=True,aliases=['PIC12C508-I/SM', 'PIC12C508-I/JW'],pins=[
            Pin(num='1',name='VDD',func=Pin.PWRIN,do_erc=True),
            Pin(num='2',name='CLKIN/OSC1/GP5',func=Pin.BIDIR,do_erc=True),
            Pin(num='3',name='OSC2/GP4',func=Pin.BIDIR,do_erc=True),
            Pin(num='4',name='Vpp/~MCLR~/GP3',do_erc=True),
            Pin(num='5',name='T0CKI/GP2',func=Pin.BIDIR,do_erc=True),
            Pin(num='6',name='ICSPCLK/GP1',func=Pin.BIDIR,do_erc=True),
            Pin(num='7',name='ICSPDAT/GP0',func=Pin.BIDIR,do_erc=True),
            Pin(num='8',name='VSS',func=Pin.PWRIN,do_erc=True)]),
        Part(name='PIC12C508A-I/P',dest=TEMPLATE,tool=SKIDL,keywords='8-Bit CMOS Microcontroller',description='PIC12C508A, 512W EPROM, 25B SRAM, SO8',ref_prefix='U',num_units=1,do_erc=True,aliases=['PIC12C508A-I/SN', 'PIC12C508A-I/SM', 'PIC12C508A-I/JW'],pins=[
            Pin(num='1',name='VDD',func=Pin.PWRIN,do_erc=True),
            Pin(num='2',name='CLKIN/OSC1/GP5',func=Pin.BIDIR,do_erc=True),
            Pin(num='3',name='OSC2/GP4',func=Pin.BIDIR,do_erc=True),
            Pin(num='4',name='Vpp/~MCLR~/GP3',do_erc=True),
            Pin(num='5',name='T0CKI/GP2',func=Pin.BIDIR,do_erc=True),
            Pin(num='6',name='ICSPCLK/GP1',func=Pin.BIDIR,do_erc=True),
            Pin(num='7',name='ICSPDAT/GP0',func=Pin.BIDIR,do_erc=True),
            Pin(num='8',name='VSS',func=Pin.PWRIN,do_erc=True)]),
        Part(name='PIC12C509-I/P',dest=TEMPLATE,tool=SKIDL,keywords='8-Bit CMOS Microcontroller',description='PIC12C509, 1024W EPROM, 41B SRAM, SO8 Wide',ref_prefix='U',num_units=1,do_erc=True,aliases=['PIC12C509-I/SM', 'PIC12C509-I/JW'],pins=[
            Pin(num='1',name='VDD',func=Pin.PWRIN,do_erc=True),
            Pin(num='2',name='CLKIN/OSC1/GP5',func=Pin.BIDIR,do_erc=True),
            Pin(num='3',name='OSC2/GP4',func=Pin.BIDIR,do_erc=True),
            Pin(num='4',name='Vpp/~MCLR~/GP3',do_erc=True),
            Pin(num='5',name='T0CKI/GP2',func=Pin.BIDIR,do_erc=True),
            Pin(num='6',name='ICSPCLK/GP1',func=Pin.BIDIR,do_erc=True),
            Pin(num='7',name='ICSPDAT/GP0',func=Pin.BIDIR,do_erc=True),
            Pin(num='8',name='VSS',func=Pin.PWRIN,do_erc=True)]),
        Part(name='PIC12C509A-I/P',dest=TEMPLATE,tool=SKIDL,keywords='8-Bit CMOS Microcontroller',description='PIC12C509A, 1024W EPROM, 41B SRAM, SO8',ref_prefix='U',num_units=1,do_erc=True,aliases=['PIC12C509A-I/SN', 'PIC12C509A-I/SM', 'PIC12C509A-I/JW'],pins=[
            Pin(num='1',name='VDD',func=Pin.PWRIN,do_erc=True),
            Pin(num='2',name='CLKIN/OSC1/GP5',func=Pin.BIDIR,do_erc=True),
            Pin(num='3',name='OSC2/GP4',func=Pin.BIDIR,do_erc=True),
            Pin(num='4',name='Vpp/~MCLR~/GP3',do_erc=True),
            Pin(num='5',name='T0CKI/GP2',func=Pin.BIDIR,do_erc=True),
            Pin(num='6',name='ICSPCLK/GP1',func=Pin.BIDIR,do_erc=True),
            Pin(num='7',name='ICSPDAT/GP0',func=Pin.BIDIR,do_erc=True),
            Pin(num='8',name='VSS',func=Pin.PWRIN,do_erc=True)]),
        Part(name='PIC12C671-I/P',dest=TEMPLATE,tool=SKIDL,keywords='8-Bit CMOS Microcontroller',description='PIC12C671, 1024W EPROM, 128B SRAM, SO8',ref_prefix='U',num_units=1,do_erc=True,aliases=['PIC12C671-I/SN', 'PIC12C671-I/JW'],pins=[
            Pin(num='1',name='VDD',func=Pin.PWRIN,do_erc=True),
            Pin(num='2',name='CLKIN/OSC1/GP5',func=Pin.BIDIR,do_erc=True),
            Pin(num='3',name='CLKOUT/OSC2/AN3/GP4',func=Pin.BIDIR,do_erc=True),
            Pin(num='4',name='Vpp/~MCLR~/GP3',do_erc=True),
            Pin(num='5',name='INT/T0CKI/AN2/GP2',func=Pin.BIDIR,do_erc=True),
            Pin(num='6',name='Vref/AN1/GP1',func=Pin.BIDIR,do_erc=True),
            Pin(num='7',name='AN0/GP0',func=Pin.BIDIR,do_erc=True),
            Pin(num='8',name='VSS',func=Pin.PWRIN,do_erc=True)]),
        Part(name='PIC12C672-I/P',dest=TEMPLATE,tool=SKIDL,keywords='8-Bit CMOS Microcontroller',description='PIC12C672, 2048W EPROM, 128B SRAM, SO8',ref_prefix='U',num_units=1,do_erc=True,aliases=['PIC12C672-I/SN', 'PIC12C672-I/JW'],pins=[
            Pin(num='1',name='VDD',func=Pin.PWRIN,do_erc=True),
            Pin(num='2',name='CLKIN/OSC1/GP5',func=Pin.BIDIR,do_erc=True),
            Pin(num='3',name='CLKOUT/OSC2/AN3/GP4',func=Pin.BIDIR,do_erc=True),
            Pin(num='4',name='Vpp/~MCLR~/GP3',do_erc=True),
            Pin(num='5',name='INT/T0CKI/AN2/GP2',func=Pin.BIDIR,do_erc=True),
            Pin(num='6',name='Vref/AN1/GP1',func=Pin.BIDIR,do_erc=True),
            Pin(num='7',name='AN0/GP0',func=Pin.BIDIR,do_erc=True),
            Pin(num='8',name='VSS',func=Pin.PWRIN,do_erc=True)]),
        Part(name='PIC12CE518-I/P',dest=TEMPLATE,tool=SKIDL,keywords='8-Bit CMOS Microcontroller',description='PIC12CE518, 512W EPROM, 25B SRAM, SO8',ref_prefix='U',num_units=1,do_erc=True,aliases=['PIC12CE518-I/SN', 'PIC12CE518-I/SM', 'PIC12CE518-I/JW'],pins=[
            Pin(num='1',name='VDD',func=Pin.PWRIN,do_erc=True),
            Pin(num='2',name='OSC1/CLKIN/GP5',func=Pin.BIDIR,do_erc=True),
            Pin(num='3',name='OSC2/GP4',func=Pin.BIDIR,do_erc=True),
            Pin(num='4',name='Vpp/~MCLR~/GP3',do_erc=True),
            Pin(num='5',name='T0CKI/GP2',func=Pin.BIDIR,do_erc=True),
            Pin(num='6',name='ICSPCLK/GP1',func=Pin.BIDIR,do_erc=True),
            Pin(num='7',name='ICSPDAT/GP0',func=Pin.BIDIR,do_erc=True),
            Pin(num='8',name='VSS',func=Pin.PWRIN,do_erc=True)]),
        Part(name='PIC12CE519-I/P',dest=TEMPLATE,tool=SKIDL,keywords='8-Bit CMOS Microcontroller',description='PIC12CE519, 1024W EPROM, 41B SRAM, SO8',ref_prefix='U',num_units=1,do_erc=True,aliases=['PIC12CE519-I/SN', 'PIC12CE519-I/SM', 'PIC12CE519-I/JW'],pins=[
            Pin(num='1',name='VDD',func=Pin.PWRIN,do_erc=True),
            Pin(num='2',name='CLKIN/OSC1/GP5',func=Pin.BIDIR,do_erc=True),
            Pin(num='3',name='OSC2/GP4',func=Pin.BIDIR,do_erc=True),
            Pin(num='4',name='Vpp/~MCLR~/GP3',do_erc=True),
            Pin(num='5',name='T0CKI/GP2',func=Pin.BIDIR,do_erc=True),
            Pin(num='6',name='ICSPCLK/GP1',func=Pin.BIDIR,do_erc=True),
            Pin(num='7',name='ICSPDAT/GP0',func=Pin.BIDIR,do_erc=True),
            Pin(num='8',name='VSS',func=Pin.PWRIN,do_erc=True)]),
        Part(name='PIC12CE673-I/P',dest=TEMPLATE,tool=SKIDL,keywords='8-Bit CMOS Microcontroller',description='1024W EPROM, 128B SRAM, 128B EPROM, PDIP8',ref_prefix='U',num_units=1,do_erc=True,aliases=['PIC12CE673-I/JW'],pins=[
            Pin(num='1',name='VDD',func=Pin.PWRIN,do_erc=True),
            Pin(num='2',name='CLKIN/OSC1/GP5',func=Pin.BIDIR,do_erc=True),
            Pin(num='3',name='CLKOUT/OSC2/AN3/GP4',func=Pin.BIDIR,do_erc=True),
            Pin(num='4',name='Vpp/~MCLR~/GP3',do_erc=True),
            Pin(num='5',name='INT/T0CKI/AN2/GP2',func=Pin.BIDIR,do_erc=True),
            Pin(num='6',name='Vref/AN1/GP1',func=Pin.BIDIR,do_erc=True),
            Pin(num='7',name='AN0/GP0',func=Pin.BIDIR,do_erc=True),
            Pin(num='8',name='VSS',func=Pin.PWRIN,do_erc=True)]),
        Part(name='PIC12CE674-I/P',dest=TEMPLATE,tool=SKIDL,keywords='8-Bit CMOS Microcontroller',description='2048W EPROM, 128B SRAM, 128B EPROM, PDIP8',ref_prefix='U',num_units=1,do_erc=True,aliases=['PIC12CE674-I/JW'],pins=[
            Pin(num='1',name='VDD',func=Pin.PWRIN,do_erc=True),
            Pin(num='2',name='CLKIN/OSC1/GP5',func=Pin.BIDIR,do_erc=True),
            Pin(num='3',name='CLKOUT/OSC2/AN3/GP4',func=Pin.BIDIR,do_erc=True),
            Pin(num='4',name='Vpp/~MCLR~/GP3',do_erc=True),
            Pin(num='5',name='INT/T0CKI/AN2/GP2',func=Pin.BIDIR,do_erc=True),
            Pin(num='6',name='Vref/AN1/GP1',func=Pin.BIDIR,do_erc=True),
            Pin(num='7',name='AN0/GP0',func=Pin.BIDIR,do_erc=True),
            Pin(num='8',name='VSS',func=Pin.PWRIN,do_erc=True)]),
        Part(name='PIC12CR509A-I/P',dest=TEMPLATE,tool=SKIDL,keywords='8-Bit CMOS Microcontroller',description='PIC12CR509A, 1024W ROM, 41B SRAM, SO8',ref_prefix='U',num_units=1,do_erc=True,aliases=['PIC12CR509A-I/SM', 'PIC12CR509A-I/SN'],pins=[
            Pin(num='1',name='VDD',func=Pin.PWRIN,do_erc=True),
            Pin(num='2',name='CLKIN/OSC1/GP5',func=Pin.BIDIR,do_erc=True),
            Pin(num='3',name='OSC2/GP4',func=Pin.BIDIR,do_erc=True),
            Pin(num='4',name='Vpp/~MCLR~/GP3',do_erc=True),
            Pin(num='5',name='T0CKI/GP2',func=Pin.BIDIR,do_erc=True),
            Pin(num='6',name='ICSPCLK/GP1',func=Pin.BIDIR,do_erc=True),
            Pin(num='7',name='ICSPDAT/GP0',func=Pin.BIDIR,do_erc=True),
            Pin(num='8',name='VSS',func=Pin.PWRIN,do_erc=True)]),
        Part(name='PIC12F508-I/P',dest=TEMPLATE,tool=SKIDL,keywords='FLASH-Based 8-Bit CMOS Microcontroller',description='PIC12F508, 512W Flash, 25B SRAM, SO8',ref_prefix='U',num_units=1,do_erc=True,aliases=['PIC12F508-I/SN', 'PIC12F508-I/MS', 'PIC12F508-I/MC'],pins=[
            Pin(num='1',name='VDD',func=Pin.PWRIN,do_erc=True),
            Pin(num='2',name='CLKIN/OSC1/GP5',func=Pin.BIDIR,do_erc=True),
            Pin(num='3',name='OSC2/GP4',func=Pin.BIDIR,do_erc=True),
            Pin(num='4',name='Vpp/~MCLR~/GP3',do_erc=True),
            Pin(num='5',name='T0CKI/GP2',func=Pin.BIDIR,do_erc=True),
            Pin(num='6',name='ICSPCLK/GP1',func=Pin.BIDIR,do_erc=True),
            Pin(num='7',name='ICSPDAT/GP0',func=Pin.BIDIR,do_erc=True),
            Pin(num='8',name='VSS',func=Pin.PWRIN,do_erc=True)]),
        Part(name='PIC12F509-I/P',dest=TEMPLATE,tool=SKIDL,keywords='FLASH-Based 8-Bit CMOS Microcontroller',description='PIC12F509, 1024W Flash, 41B SRAM, SO8',ref_prefix='U',num_units=1,do_erc=True,aliases=['PIC12F509-I/SN', 'PIC12F509-I/MS', 'PIC12F509-I/MC'],pins=[
            Pin(num='1',name='VDD',func=Pin.PWRIN,do_erc=True),
            Pin(num='2',name='CLKIN/OSC1/GP5',func=Pin.BIDIR,do_erc=True),
            Pin(num='3',name='OSC2/GP4',func=Pin.BIDIR,do_erc=True),
            Pin(num='4',name='Vpp/~MCLR~/GP3',do_erc=True),
            Pin(num='5',name='T0CKI/GP2',func=Pin.BIDIR,do_erc=True),
            Pin(num='6',name='ICSPCLK/GP1',func=Pin.BIDIR,do_erc=True),
            Pin(num='7',name='ICSPDAT/GP0',func=Pin.BIDIR,do_erc=True),
            Pin(num='8',name='VSS',func=Pin.PWRIN,do_erc=True)]),
        Part(name='PIC12F510-I/P',dest=TEMPLATE,tool=SKIDL,keywords='FLASH-Based 8-Bit CMOS Microcontroller',description='PIC12F510, 1024W FLASH, 38B SRAM, SO8',ref_prefix='U',num_units=1,do_erc=True,aliases=['PIC12F510-I/SN', 'PIC12F510-I/MS', 'PIC12F510-I/MC'],pins=[
            Pin(num='1',name='VDD',func=Pin.PWRIN,do_erc=True),
            Pin(num='2',name='CLKIN/OSC1/GP5',func=Pin.BIDIR,do_erc=True),
            Pin(num='3',name='OSC2/GP4',func=Pin.BIDIR,do_erc=True),
            Pin(num='4',name='Vpp/~MCLR~/GP3',do_erc=True),
            Pin(num='5',name='T0CKI/C1OUTI/AN2/GP2',func=Pin.BIDIR,do_erc=True),
            Pin(num='6',name='ICSPCLK/C1IN-/AN1/GP1',func=Pin.BIDIR,do_erc=True),
            Pin(num='7',name='ICSPDAT/C1IN+/AN0/GP0',func=Pin.BIDIR,do_erc=True),
            Pin(num='8',name='VSS',func=Pin.PWRIN,do_erc=True)]),
        Part(name='PIC12F519-I/P',dest=TEMPLATE,tool=SKIDL,keywords='FLASH-Based 8-Bit CMOS Microcontroller',description='PIC12F519, 1024W Flash, 41B SRAM, 64B EEPROM, SO8',ref_prefix='U',num_units=1,do_erc=True,aliases=['PIC12F519-I/SN', 'PIC12F519-I/MS', 'PIC12F519-I/MC'],pins=[
            Pin(num='1',name='VDD',func=Pin.PWRIN,do_erc=True),
            Pin(num='2',name='CLKIN/OSC1/GP5',func=Pin.BIDIR,do_erc=True),
            Pin(num='3',name='OSC2/GP4',func=Pin.BIDIR,do_erc=True),
            Pin(num='4',name='Vpp/~MCLR~/GP3',do_erc=True),
            Pin(num='5',name='T0CKI/GP2',func=Pin.BIDIR,do_erc=True),
            Pin(num='6',name='ICSPCLK/GP1',func=Pin.BIDIR,do_erc=True),
            Pin(num='7',name='ICSPDAT/GP0',func=Pin.BIDIR,do_erc=True),
            Pin(num='8',name='VSS',func=Pin.PWRIN,do_erc=True)]),
        Part(name='PIC12F609-I/P',dest=TEMPLATE,tool=SKIDL,keywords='FLASH-Based 8-Bit CMOS Microcontroller',description='PIC12F609, 1024W Flash, 64B SRAM, SO8',ref_prefix='U',num_units=1,do_erc=True,aliases=['PIC12F609-I/SN', 'PIC12F609-I/MS', 'PIC12F609-I/MC'],pins=[
            Pin(num='1',name='VDD',func=Pin.PWRIN,do_erc=True),
            Pin(num='2',name='T1CKI/CLKIN/OSC1/GP5',func=Pin.BIDIR,do_erc=True),
            Pin(num='3',name='CIN1-/~T1G~/CLKOUT/OSC2/GP4',func=Pin.BIDIR,do_erc=True),
            Pin(num='4',name='Vpp/~MCLR~/GP3',do_erc=True),
            Pin(num='5',name='T0CKI/INT/COUT/GP2',func=Pin.BIDIR,do_erc=True),
            Pin(num='6',name='CIN0-/ICSPCLK/GP1',func=Pin.BIDIR,do_erc=True),
            Pin(num='7',name='CIN+/ICSPDAT/GP0',func=Pin.BIDIR,do_erc=True),
            Pin(num='8',name='VSS',func=Pin.PWRIN,do_erc=True)]),
        Part(name='PIC12F615-I/P',dest=TEMPLATE,tool=SKIDL,keywords='FLASH-Based 8-Bit CMOS Microcontroller',description='PIC12F615, 1024W Flash, 64B SRAM, SO8',ref_prefix='U',num_units=1,do_erc=True,aliases=['PIC12F615-I/SN', 'PIC12F615-I/MS', 'PIC12F615-I/MC'],pins=[
            Pin(num='1',name='VDD',func=Pin.PWRIN,do_erc=True),
            Pin(num='2',name='T1CKI/P1A/CLKIN/OSC1/GP5',func=Pin.BIDIR,do_erc=True),
            Pin(num='3',name='AN3/CIN1-/~T1G~/P1B/CLKOUT/OSC2/GP4',func=Pin.BIDIR,do_erc=True),
            Pin(num='4',name='Vpp/~MCLR~/~T1G~/GP3',do_erc=True),
            Pin(num='5',name='AN2/T0CKI/INT/COUT/CCP1/P1A/GP2',func=Pin.BIDIR,do_erc=True),
            Pin(num='6',name='AN1/CIN0-/Vref/ICSPCLK/GP1',func=Pin.BIDIR,do_erc=True),
            Pin(num='7',name='AN0/CIN+/P1B/ICSPDAT/GP0',func=Pin.BIDIR,do_erc=True),
            Pin(num='8',name='VSS',func=Pin.PWRIN,do_erc=True)]),
        Part(name='PIC12F617-I/P',dest=TEMPLATE,tool=SKIDL,keywords='FLASH-Based 8-Bit CMOS Microcontroller',description='PIC12F617, 2048W Flash, 128B SRAM, SO8',ref_prefix='U',num_units=1,do_erc=True,aliases=['PIC12F617-I/SN', 'PIC12F617-I/MS', 'PIC12F617-I/MC'],pins=[
            Pin(num='1',name='VDD',func=Pin.PWRIN,do_erc=True),
            Pin(num='2',name='T1CKI/P1A/CLKIN/OSC1/GP5',func=Pin.BIDIR,do_erc=True),
            Pin(num='3',name='AN3/CIN1-/~T1G~/P1B/CLKOUT/OSC2/GP4',func=Pin.BIDIR,do_erc=True),
            Pin(num='4',name='Vpp/~MCLR~/~T1G~/GP3',do_erc=True),
            Pin(num='5',name='AN2/T0CKI/INT/COUT/CCP1/P1A/GP2',func=Pin.BIDIR,do_erc=True),
            Pin(num='6',name='AN1/CIN0-/Vref/ICSPCLK/GP1',func=Pin.BIDIR,do_erc=True),
            Pin(num='7',name='AN0/CIN+/P1B/ICSPDAT/GP0',func=Pin.BIDIR,do_erc=True),
            Pin(num='8',name='VSS',func=Pin.PWRIN,do_erc=True)]),
        Part(name='PIC12F629-I/P',dest=TEMPLATE,tool=SKIDL,keywords='FLASH-Based 8-Bit CMOS Microcontroller',description='PIC12F629, 1024W Flash, 64B SRAM, 128B EEPROM, SO8',ref_prefix='U',num_units=1,do_erc=True,aliases=['PIC12F629-I/SN', 'PIC12F629-I/MS', 'PIC12F629-I/MC'],pins=[
            Pin(num='1',name='VDD',func=Pin.PWRIN,do_erc=True),
            Pin(num='2',name='T1CKI/CLKIN/OSC1/GP5',func=Pin.BIDIR,do_erc=True),
            Pin(num='3',name='~T1G~/CLKOUT/OSC2/GP4',func=Pin.BIDIR,do_erc=True),
            Pin(num='4',name='Vpp/~MCLR~/GP3',do_erc=True),
            Pin(num='5',name='T0CKI/INT/COUT/GP2',func=Pin.BIDIR,do_erc=True),
            Pin(num='6',name='CIN-/ICSPCLK/GP1',func=Pin.BIDIR,do_erc=True),
            Pin(num='7',name='CIN+/ICSPDAT/GP0',func=Pin.BIDIR,do_erc=True),
            Pin(num='8',name='VSS',func=Pin.PWRIN,do_erc=True)]),
        Part(name='PIC12F635-I/P',dest=TEMPLATE,tool=SKIDL,keywords='FLASH-Based 8-Bit CMOS Microcontroller',description='PIC12F635, 1024W Flash, 64B SRAM, 128B EEPROM, SO8',ref_prefix='U',num_units=1,do_erc=True,aliases=['PIC12F635-I/SN', 'PIC12F635-I/MS', 'PIC12F635-I/MC'],pins=[
            Pin(num='1',name='VDD',func=Pin.PWRIN,do_erc=True),
            Pin(num='2',name='T1CKI/CLKIN/OSC1/GP5',func=Pin.BIDIR,do_erc=True),
            Pin(num='3',name='AN3/~T1G~/CLKOUT/OSC2/GP4',func=Pin.BIDIR,do_erc=True),
            Pin(num='4',name='Vpp/~MCLR~/GP3',do_erc=True),
            Pin(num='5',name='AN2/T0CKI/INT/COUT/GP2',func=Pin.BIDIR,do_erc=True),
            Pin(num='6',name='AN1/CIN-/Vref/ICSPCLK/GP1',func=Pin.BIDIR,do_erc=True),
            Pin(num='7',name='AN0/CIN+/ICSPDAT/GP0',func=Pin.BIDIR,do_erc=True),
            Pin(num='8',name='VSS',func=Pin.PWRIN,do_erc=True)]),
        Part(name='PIC12F675-I/P',dest=TEMPLATE,tool=SKIDL,keywords='FLASH-Based 8-Bit CMOS Microcontroller',description='PIC12F675, 1024W Flash, 64B SRAM, 128B EEPROM, SO8',ref_prefix='U',num_units=1,do_erc=True,aliases=['PIC12F675-I/SN', 'PIC12F675-I/MS', 'PIC12F675-I/MC'],pins=[
            Pin(num='1',name='VDD',func=Pin.PWRIN,do_erc=True),
            Pin(num='2',name='T1CKI/CLKIN/OSC1/GP5',func=Pin.BIDIR,do_erc=True),
            Pin(num='3',name='AN3/~T1G~/CLKOUT/OSC2/GP4',func=Pin.BIDIR,do_erc=True),
            Pin(num='4',name='Vpp/~MCLR~/GP3',do_erc=True),
            Pin(num='5',name='AN2/T0CKI/INT/COUT/GP2',func=Pin.BIDIR,do_erc=True),
            Pin(num='6',name='AN1/CIN-/Vref/ICSPCLK/GP1',func=Pin.BIDIR,do_erc=True),
            Pin(num='7',name='AN0/CIN+/ICSPDAT/GP0',func=Pin.BIDIR,do_erc=True),
            Pin(num='8',name='VSS',func=Pin.PWRIN,do_erc=True)]),
        Part(name='PIC12F683-I/P',dest=TEMPLATE,tool=SKIDL,keywords='FLASH-Based 8-Bit CMOS Microcontroller',description='PIC12F683, 2048W Flash, 128B SRAM, 256B EEPROM, SO8',ref_prefix='U',num_units=1,do_erc=True,aliases=['PIC12F683-I/SN', 'PIC12F683-I/MS', 'PIC12F683-I/MC'],pins=[
            Pin(num='1',name='VDD',func=Pin.PWRIN,do_erc=True),
            Pin(num='2',name='T1CKI/CLKIN/OSC1/GP5',func=Pin.BIDIR,do_erc=True),
            Pin(num='3',name='AN3/~T1G~/CLKOUT/OSC2/GP4',func=Pin.BIDIR,do_erc=True),
            Pin(num='4',name='Vpp/~MCLR~/GP3',do_erc=True),
            Pin(num='5',name='AN2/T0CKI/INT/COUT/CCCP1/GP2',func=Pin.BIDIR,do_erc=True),
            Pin(num='6',name='AN1/CIN-/Vref/ICSPCLK/GP1',func=Pin.BIDIR,do_erc=True),
            Pin(num='7',name='AN0/CIN+/ICSPDAT/ULPWU/GP0',func=Pin.BIDIR,do_erc=True),
            Pin(num='8',name='VSS',func=Pin.PWRIN,do_erc=True)]),
        Part(name='PIC12F752-I/P',dest=TEMPLATE,tool=SKIDL,keywords='FLASH-Based 8-Bit CMOS Microcontroller',description='PIC12F752, 1024W Flash, 64B SRAM, SO8',ref_prefix='U',num_units=1,do_erc=True,aliases=['PIC12F752-I/SN', 'PIC12F752-I/MC'],pins=[
            Pin(num='1',name='VDD',func=Pin.PWRIN,do_erc=True),
            Pin(num='2',name='CLKIN/OSC1/GP5',func=Pin.BIDIR,do_erc=True),
            Pin(num='3',name='OSC2/GP4',func=Pin.BIDIR,do_erc=True),
            Pin(num='4',name='Vpp/~MCLR~/GP3',do_erc=True),
            Pin(num='5',name='T0CKI/GP2',func=Pin.BIDIR,do_erc=True),
            Pin(num='6',name='ICSPCLK/GP1',func=Pin.BIDIR,do_erc=True),
            Pin(num='7',name='ICSPDAT/GP0',func=Pin.BIDIR,do_erc=True),
            Pin(num='8',name='VSS',func=Pin.PWRIN,do_erc=True)]),
        Part(name='PIC12HV609-I/P',dest=TEMPLATE,tool=SKIDL,keywords='FLASH-Based 8-Bit CMOS Microcontroller High Voltage',description='PIC12HV609, 1024W Flash, 64B SRAM, SO8',ref_prefix='U',num_units=1,do_erc=True,aliases=['PIC12HV609-I/SN', 'PIC12HV609-I/MS', 'PIC12HV609-I/MC'],pins=[
            Pin(num='1',name='VDD',func=Pin.PWRIN,do_erc=True),
            Pin(num='2',name='T1CKI/CLKIN/OSC1/GP5',func=Pin.BIDIR,do_erc=True),
            Pin(num='3',name='CIN1-/~T1G~/CLKOUT/OSC2/GP4',func=Pin.BIDIR,do_erc=True),
            Pin(num='4',name='Vpp/~MCLR~/GP3',do_erc=True),
            Pin(num='5',name='T0CKI/INT/COUT/GP2',func=Pin.BIDIR,do_erc=True),
            Pin(num='6',name='CIN0-/ICSPCLK/GP1',func=Pin.BIDIR,do_erc=True),
            Pin(num='7',name='CIN+/ICSPDAT/GP0',func=Pin.BIDIR,do_erc=True),
            Pin(num='8',name='VSS',func=Pin.PWRIN,do_erc=True)]),
        Part(name='PIC12HV615-I/P',dest=TEMPLATE,tool=SKIDL,keywords='FLASH-Based 8-Bit CMOS Microcontroller High Voltage',description='PIC12HV615, 1024W Flash, 64B SRAM, SO8',ref_prefix='U',num_units=1,do_erc=True,aliases=['PIC12HV615-I/SN', 'PIC12HV615-I/MS', 'PIC12HV615-I/MC'],pins=[
            Pin(num='1',name='VDD',func=Pin.PWRIN,do_erc=True),
            Pin(num='2',name='T1CKI/P1A/CLKIN/OSC1/GP5',func=Pin.BIDIR,do_erc=True),
            Pin(num='3',name='AN3/CIN1-/~T1G~/P1B/CLKOUT/OSC2/GP4',func=Pin.BIDIR,do_erc=True),
            Pin(num='4',name='Vpp/~MCLR~/~T1G~/GP3',do_erc=True),
            Pin(num='5',name='AN2/T0CKI/INT/COUT/CCP1/P1A/GP2',func=Pin.BIDIR,do_erc=True),
            Pin(num='6',name='AN1/CIN0-/Vref/ICSPCLK/GP1',func=Pin.BIDIR,do_erc=True),
            Pin(num='7',name='AN0/CIN+/P1B/ICSPDAT/GP0',func=Pin.BIDIR,do_erc=True),
            Pin(num='8',name='VSS',func=Pin.PWRIN,do_erc=True)]),
        Part(name='PIC12HV752-I/P',dest=TEMPLATE,tool=SKIDL,keywords='FLASH-Based 8-Bit CMOS Microcontroller High Voltage',description='PIC12HV752, 1024W Flash, 64B SRAM, SO8',ref_prefix='U',num_units=1,do_erc=True,aliases=['PIC12HV752-I/SN', 'PIC12HV752-I/MC'],pins=[
            Pin(num='1',name='VDD',func=Pin.PWRIN,do_erc=True),
            Pin(num='2',name='CLKIN/OSC1/GP5',func=Pin.BIDIR,do_erc=True),
            Pin(num='3',name='OSC2/GP4',func=Pin.BIDIR,do_erc=True),
            Pin(num='4',name='Vpp/~MCLR~/GP3',do_erc=True),
            Pin(num='5',name='T0CKI/GP2',func=Pin.BIDIR,do_erc=True),
            Pin(num='6',name='ICSPCLK/GP1',func=Pin.BIDIR,do_erc=True),
            Pin(num='7',name='ICSPDAT/GP0',func=Pin.BIDIR,do_erc=True),
            Pin(num='8',name='VSS',func=Pin.PWRIN,do_erc=True)]),
        Part(name='PIC12LF1840T48-I/ST',dest=TEMPLATE,tool=SKIDL,keywords='FLASH-Based 8-Bit CMOS Microcontroller XLP RF Transmitter',description='PIC12LF1840T48, 4kW FLASH, 256B SRAM, 256B EEPROM, RF, TSSOP14',ref_prefix='U',num_units=1,do_erc=True,pins=[
            Pin(num='1',name='VDD',func=Pin.PWRIN,do_erc=True),
            Pin(num='2',name='RX/DT/CCP1/P1A/SRNQ/T1CKI/T1OSI/CLKIN/OSC1/RA5',func=Pin.BIDIR,do_erc=True),
            Pin(num='3',name='MDCIN2/T1G/P1B/TX/CK/SDO/CLKR/C1IN1-/T1OSO/CPS3/AN3/CLKOUT/OSC2/RA4',func=Pin.BIDIR,do_erc=True),
            Pin(num='4',name='Vpp/~MCLR~/T1G/~SS~/RA3',do_erc=True),
            Pin(num='5',name='VDDRF',func=Pin.PWRIN,do_erc=True),
            Pin(num='6',name='CTRL',do_erc=True),
            Pin(num='7',name='RFOUT',do_erc=True),
            Pin(num='8',name='VSSRF',func=Pin.PWRIN,do_erc=True),
            Pin(num='9',name='DATA',do_erc=True),
            Pin(num='10',name='XTAL',do_erc=True),
            Pin(num='11',name='AN2/CPS2/C1OUT/SRQ/T0CKI/CCP1/P1A/FLT0/SDA/SDI/INT/MDCIN1/RA2',func=Pin.BIDIR,do_erc=True),
            Pin(num='12',name='AN1/CPS1/Vref/C1IN0-/SRI/RX/DT/SCL/SCK/MDMIN/ICSPCLK/RA1',func=Pin.BIDIR,do_erc=True),
            Pin(num='13',name='AN0/CPS0/C1IN+/DACOUT/TX/CK/SDO/~SS~/P1B/MDOUT/ICSPDAT/RA0',func=Pin.BIDIR,do_erc=True),
            Pin(num='14',name='VSS',func=Pin.PWRIN,do_erc=True)])])


# -*- coding: utf-8 -*-

# MIT license
#
# Copyright (C) 2018 by XESS Corp.
#
# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documentation files (the "Software"), to deal
# in the Software without restriction, including without limitation the rights
# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice shall be included in
# all copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
# THE SOFTWARE.

"""
Handles part pins.
"""

from __future__ import unicode_literals
from __future__ import print_function
from __future__ import division
from __future__ import absolute_import

from builtins import super
from builtins import range
from future import standard_library
standard_library.install_aliases()
from copy import copy

from .utilities import *


class Pin(object):
    """
    A class for storing data about pins for a part.

    Args:
        attribs: Key/value pairs of attributes to add to the library.

    Attributes:
        nets: The electrical nets this pin is connected to (can be >1).
        part: Link to the Part object this pin belongs to.
        do_erc: When false, the pin is not checked for ERC violations.
    """

    # Various types of pins.
    INPUT, OUTPUT, BIDIR, TRISTATE, PASSIVE, UNSPEC, PWRIN,\
    PWROUT, OPENCOLL, OPENEMIT, NOCONNECT = range(11)

    # Various drive levels a pin can output:
    #   NOCONNECT_DRIVE: NC pin drive.
    #   NO_DRIVE: No drive capability (like an input pin).
    #   PASSIVE_DRIVE: Small drive capability, such as a pullup.
    #   ONESIDE_DRIVE: Can pull high (open-emitter) or low (open-collector).
    #   TRISTATE_DRIVE: Can pull high/low and be in high-impedance state.
    #   PUSHPULL_DRIVE: Can actively drive high or low.
    #   POWER_DRIVE: A power supply or ground line.
    NOCONNECT_DRIVE, NO_DRIVE, PASSIVE_DRIVE, ONESIDE_DRIVE,\
    TRISTATE_DRIVE, PUSHPULL_DRIVE, POWER_DRIVE = range(7)

    # Information about the various types of pins:
    #   function: A string describing the pin's function.
    #   drive: The drive capability of the pin.
    #   rcv_min: The minimum amount of drive the pin must receive to function.
    #   rcv_max: The maximum amount of drive the pin can receive and still function.
    pin_info = {
        INPUT: {
            'function': 'INPUT',
            'func_str': 'INPUT',
            'drive': NO_DRIVE,
            'max_rcv': POWER_DRIVE,
            'min_rcv': PASSIVE_DRIVE,
        },
        OUTPUT: {
            'function': 'OUTPUT',
            'func_str': 'OUTPUT',
            'drive': PUSHPULL_DRIVE,
            'max_rcv': PASSIVE_DRIVE,
            'min_rcv': NO_DRIVE,
        },
        BIDIR: {
            'function': 'BIDIRECTIONAL',
            'func_str': 'BIDIR',
            'drive': TRISTATE_DRIVE,
            'max_rcv': POWER_DRIVE,
            'min_rcv': NO_DRIVE,
        },
        TRISTATE: {
            'function': 'TRISTATE',
            'func_str': 'TRISTATE',
            'drive': TRISTATE_DRIVE,
            'max_rcv': TRISTATE_DRIVE,
            'min_rcv': NO_DRIVE,
        },
        PASSIVE: {
            'function': 'PASSIVE',
            'func_str': 'PASSIVE',
            'drive': PASSIVE_DRIVE,
            'max_rcv': POWER_DRIVE,
            'min_rcv': NO_DRIVE,
        },
        UNSPEC: {
            'function': 'UNSPECIFIED',
            'func_str': 'UNSPEC',
            'drive': NO_DRIVE,
            'max_rcv': POWER_DRIVE,
            'min_rcv': NO_DRIVE,
        },
        PWRIN: {
            'function': 'POWER-IN',
            'func_str': 'PWRIN',
            'drive': NO_DRIVE,
            'max_rcv': POWER_DRIVE,
            'min_rcv': POWER_DRIVE,
        },
        PWROUT: {
            'function': 'POWER-OUT',
            'func_str': 'PWROUT',
            'drive': POWER_DRIVE,
            'max_rcv': PASSIVE_DRIVE,
            'min_rcv': NO_DRIVE,
        },
        OPENCOLL: {
            'function': 'OPEN-COLLECTOR',
            'func_str': 'OPENCOLL',
            'drive': ONESIDE_DRIVE,
            'max_rcv': TRISTATE_DRIVE,
            'min_rcv': NO_DRIVE,
        },
        OPENEMIT: {
            'function': 'OPEN-EMITTER',
            'func_str': 'OPENEMIT',
            'drive': ONESIDE_DRIVE,
            'max_rcv': TRISTATE_DRIVE,
            'min_rcv': NO_DRIVE,
        },
        NOCONNECT: {
            'function': 'NO-CONNECT',
            'func_str': 'NOCONNECT',
            'drive': NOCONNECT_DRIVE,
            'max_rcv': NOCONNECT_DRIVE,
            'min_rcv': NOCONNECT_DRIVE,
        },
    }

    def __init__(self, **attribs):
        self.nets = []
        self.part = None
        self.name = ''
        self.num = ''
        self.do_erc = True

        # Attach additional attributes to the pin.
        for k, v in attribs.items():
            setattr(self, k, v)

    def copy(self, num_copies=1, **attribs):
        """
        Return copy or list of copies of a pin including any net connection.

        Args:
            num_copies: Number of copies to make of pin.

        Keyword Args:
            attribs: Name/value pairs for setting attributes for the pin.

        Notes:
            An instance of a pin can be copied just by calling it like so::

                p = Pin()     # Create a pin.
                p_copy = p()  # This is a copy of the pin.
        """

        # Check that a valid number of copies is requested.
        if not isinstance(num_copies, int):
            logger.error(
                "Can't make a non-integer number ({}) of copies of a pin!".
                format(num_copies))
            raise Exception
        if num_copies < 0:
            logger.error(
                "Can't make a negative number ({}) of copies of a pin!".format(
                    num_copies))
            raise Exception

        copies = []
        for _ in range(num_copies):

            # Make a shallow copy of the pin.
            cpy = copy(self)

            # The copy is not on a net, yet.
            cpy.nets = []

            # Connect the new pin to the same net as the original.
            if self.nets:
                self.nets[0] += cpy

            # Attach additional attributes to the pin.
            for k, v in attribs.items():
                setattr(cpy, k, v)

            copies.append(cpy)

        return list_or_scalar(copies)

    # Make copies with the multiplication operator or by calling the object.
    __mul__ = copy
    __rmul__ = copy
    __call__ = copy

    def is_connected(self):
        """Return true if a pin is connected to a net (but not a no-connect net)."""

        from .Net import Net, NCNet

        if not self.nets:
            # This pin is not connected to any nets.
            return False

        # Get the types of things this pin is connected to.
        net_types = set([type(n) for n in self.nets])

        if set([NCNet]) == net_types:
            # This pin is only connected to no-connect nets.
            return False
        if set([Net]) == net_types:
            # This pin is only connected to normal nets.
            return True
        if set([Net, NCNet]) == net_types:
            # Can't be connected to both normal and no-connect nets!
            logger.error(
                '{} is connected to both normal and no-connect nets!'.format(
                    self.erc_desc()))
            raise Exception
        # This is just strange...
        logger.error("{} is connected to something strange: {}.".format(
            self.erc_desc(), self.nets))
        raise Exception

    def is_attached(self, pin_net_bus):
        """Return true if this pin is attached to the given pin, net or bus."""

        from .Net import Net
        from .Pin import Pin

        if not self.is_connected():
            return False
        if isinstance(pin_net_bus, Pin):
            if pin_net_bus.is_connected():
                return pin_net_bus.net.is_attached(self.net)
            return False
        if isinstance(pin_net_bus, Net):
            return pin_net_bus.is_attached(self.net)
        if isinstance(pin_net_bus, Bus):
            for net in pin_net_bus[:]:
                if self.net.is_attached(net):
                    return True
            return False
        logger.error("Pins can't be attached to {}!".format(type(pin_net_bus)))
        raise Exception

    def connect(self, *pins_nets_buses):
        """
        Return the pin after connecting it to one or more nets or pins.

        Args:
            pins_nets_buses: One or more Pin, Net or Bus objects or
                lists/tuples of them.

        Returns:
            The updated pin with the new connections.

        Notes:
            You can connect nets or pins to a pin like so::

                p = Pin()     # Create a pin.
                n = Net()     # Create a net.
                p += net      # Connect the net to the pin.
        """

        from .Net import Net

        # Go through all the pins and/or nets and connect them to this pin.
        for pn in expand_buses(flatten(pins_nets_buses)):
            if isinstance(pn, Pin):
                # Connecting pin-to-pin.
                if self.is_connected():
                    # If self is already connected to a net, then add the
                    # other pin to the same net.
                    self.nets[0] += pn
                elif pn.is_connected():
                    # If self is unconnected but the other pin is, then
                    # connect self to the other pin's net.
                    pn.nets[0] += self
                else:
                    # Neither pin is connected to a net, so create a net
                    # in the same circuit as the pin and attach both to it.
                    Net(circuit=self.part.circuit).connect(self, pn)
            elif isinstance(pn, Net):
                # Connecting pin-to-net, so just connect the pin to the net.
                pn += self
            else:
                logger.error('Cannot attach non-Pin/non-Net {} to {}.'.format(
                    type(pn), self.erc_desc()))
                raise Exception

        # Set the flag to indicate this result came from the += operator.
        self.iadd_flag = True  # pylint: disable=attribute-defined-outside-init

        return self

    # Connect a net to a pin using the += operator.
    __iadd__ = connect

    def disconnect(self):
        """Disconnect this pin from all nets."""
        if not self.net:
            return
        for n in self.nets:
            n.disconnect(self)
        self.nets = []

    def get_nets(self):
        """Return a list containing the Net objects connected to this pin."""
        return self.nets

    def get_pins(self):
        """Return a list containing this pin."""
        return to_list(self)

    def erc_desc(self):
        """Return a string describing this pin for ERC."""
        desc = "{func} pin {num}/{name} of {part}".format(
            part=self.part.erc_desc(),
            num=self.num,
            name=self.name,
            func=Pin.pin_info[self.func]['function'])
        return desc

    def __str__(self):
        """Return a description of this pin as a string."""
        part_ref = getattr(self.part, 'ref', '???')
        pin_num = getattr(self, 'num', '???')
        pin_name = getattr(self, 'name', '???')
        pin_func = getattr(self, 'func', Pin.UNSPEC)
        pin_func_str = Pin.pin_info[pin_func]['function']
        return 'Pin {ref}/{num}/{name}/{func}'.format(
            ref=part_ref, num=pin_num, name=pin_name, func=pin_func_str)

    __repr__ = __str__

    def export(self):
        """Return a string to recreate a Pin object."""
        attribs = []
        for k in ['num', 'name', 'func', 'do_erc']:
            v = getattr(self, k, None)
            if v:
                if k == 'func':
                    # Assign the pin function using the actual name of the
                    # function, not its numerical value (in case that changes
                    # in the future if more pin functions are added).
                    v = 'Pin.' + Pin.pin_info[v]['func_str']
                else:
                    v = repr(v)
                attribs.append('{}={}'.format(k, v))
        return 'Pin({})'.format(','.join(attribs))

    @property
    def net(self):
        """Return one of the nets the pin is connected to."""
        if self.nets:
            return self.nets[0]
        return None


##############################################################################


class PhantomPin(Pin):
    """
    A pin type that exists solely to tie two pinless nets together.
    It will not participate in generating any netlists.
    """

    def __init__(self, **attribs):
        super(PhantomPin, self).__init__(**attribs)
        self.nets = []
        self.part = None
        self.do_erc = False
